library webspec_socket;
import 'package:flutter/foundation.dart';
import 'package:socket_io_client/socket_io_client.dart' as IO;

class WebspecSocket {
  late IO.Socket _socket;
  final String baseUrl;
  final String apiKey;

  WebspecSocket._private(this.baseUrl, this.apiKey) {
    _socket = IO.io(
      baseUrl,
      IO.OptionBuilder().setTransports(['websocket']).build(),
    );
    _socket.connect();
    // set up the socket with apiKey here
  }

  static WebspecSocket? _instance;

  factory WebspecSocket(String baseUrl, String apiKey) {
    _instance ??= WebspecSocket._private(baseUrl, apiKey);
    return _instance!;
  }


  void disconnect() {
    _socket.disconnect();
  }


  void receive(String event, void Function(dynamic) callback) {
    _socket.on(event, callback);
  }

  void send(String event, dynamic data) {
    _socket.emit(event, data);
  }

  void onError(void Function(dynamic) errorCallback) {
    _socket.onError(errorCallback);
  }

  void onDisconnect(Function disconnectCallback) {
    _socket.onDisconnect((_) {
      if (kDebugMode) {
        print("Disconnect");
      }
      disconnectCallback();
    });
  }

  void onConnect(Function connectCallback) {
    _socket.onConnect((_) {
      if (kDebugMode) {
        print("Connecting to ${_socket.id}");
      }
      connectCallback();
    });
  }

  void onConnectError(void Function(dynamic) connectErrorCallback) {
    _socket.onConnectError((err) {
      if (kDebugMode) {
        print("Connect Error to ${err.toString()}");
      }
      connectErrorCallback(err);
    });
  }
}
